@@ -69,10 +69,11 @@ class GSCNodeFunctionReceiver:
 
 class GSCNodeFunctionCallerSocket:
     @classmethod
-    def create(cls, sio, gsdbs, oncnodefunctionanswer):
+    def create(cls, sio, gsdbs, oncnodefunctionanswer,loadbalancerurl):
         self = GSCNodeFunctionCallerSocket()
         self.sio = sio
         self.gsdbs = gsdbs
+        self.loadbalancerurl=loadbalancerurl
         self._logger = logging.getLogger(__name__)
         self.oncnodefunctionanswer = oncnodefunctionanswer
 
@@ -89,10 +90,10 @@ class GSCNodeFunctionCallerSocket:
 
         connectURL = ""
 
-        if "localhost" in self.gsdbs.credentials["signalserver"]:
-            connectURL = f'{self.gsdbs.credentials["signalserver"]}:{str(self.gsdbs.credentials["signalport"])}'
+        if "localhost" in self.loadbalancerurl:
+            connectURL = f'{self.loadbalancerurl}'
         else:
-            connectURL = self.gsdbs.credentials["signalserver"]
+            connectURL = self.loadbalancerurl
 
         self.sio.connect(
             f'{connectURL}?gssession={self.gsdbs.cookiejar.get("session")}.{self.gsdbs.cookiejar.get("signature")}&caller=true')
@@ -100,17 +101,19 @@ class GSCNodeFunctionCallerSocket:
 
 
 class GSCNodeFunctionCaller:
-    def __init__(self, gsdbs, oncnodefunctionanswer):
+    def __init__(self, gsdbs, oncnodefunctionanswer,loadbalancerurl,poolsize):
         self.sio = socketio.Client()
         self.gsdbs = gsdbs
         self.oncnodefunctionanswer = oncnodefunctionanswer
-        self._pool = Pool(self.gsdbs.credentials["poolsize"])
+        self._pool = Pool(poolsize)
+        self.loadbalancerurl=loadbalancerurl
         self._logger = logging.getLogger(__name__)
 
     def startSocket(self):
         GSCNodeFunctionCallerSocket.create(self.sio,
                                            self.gsdbs,
-                                           self.oncnodefunctionanswer)
+                                           self.oncnodefunctionanswer,
+                                           self.loadbalancerurl)
 
     def on_success(self, r):
         self._logger.debug('cnodefunction succeed')
