@@ -5,6 +5,7 @@ import platform
 from aiortc import VideoStreamTrack
 from av import VideoFrame
 # import pyaudio as pyaudio
+from datetime import datetime
 import socketio
 from aiortc import RTCPeerConnection, RTCRtpSender, RTCConfiguration, RTCIceServer, RTCSessionDescription
 from aiortc.contrib.media import MediaPlayer, MediaRelay, MediaBlackhole
@@ -18,6 +19,7 @@ relay = None
 webcam = None
 import numpy as np
 
+_logger = logging.getLogger(__name__)
 # p = pyaudio.PyAudio()
 # stream = p.open(format=pyaudio.paFloat32,
 #                 channels=1,
@@ -48,6 +50,18 @@ async def checkTerminatorQueue(opqueue):
                 await track.write(data, 0)
 
 
+async def checkCommandQueue(commandqueue, channellist):
+    while True:
+        data = await commandqueue.get()
+        if data is not None:
+            for channel in channellist:
+                if channel.readyState == "open":
+                    channel.send(data)
+                if channel.readyState != "open" and channel.readyState != "connecting":
+                    _logger.warning("closing datachannel : " + str(channel.readyState))
+                    channellist.remove(channel)
+
+
 class TerminatorStreamTrack(VideoStreamTrack):
     """
     A video track that returns an animated flag.
@@ -303,6 +317,9 @@ class GSPeerConnectionBroadcasterUniversalNew:
         @self.sio.event
         async def broadcaster(id, description, emitter):
 
+            if emitter == "watcher":
+                return
+
             if len(self.rtconfiList) > 0:
                 pc = RTCPeerConnection(configuration=RTCConfiguration(self.rtconfiList))
             else:
@@ -347,33 +364,30 @@ class GSPeerConnectionBroadcasterUniversalNew:
                 video_tracklist.append(terminatorviewtrack)
                 pc.addTrack(terminatorviewtrack)
 
-            # if self.addChannel is not None:
-            #     sendchannel = pc.createDataChannel("terminatorview")
-            #     self.addChannel(sendchannel)
-            #     channel_log(self._logger, sendchannel, "-", "created by local party")
-            #
-            #     @sendchannel.on("open")
-            #     def on_open():
-            #         self._logger.info("datachannel open")
+            if self.addChannel is not None:
+                sendchannel = pc.createDataChannel("terminatorview")
+                self.addChannel(sendchannel)
+                channel_log(self._logger, sendchannel, "-", "created by local party")
 
-            # @sendchannel.on("message")
-            # def on_message(message):
-            #     def subprocess_cmd(command):
-            #         process = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
-            #         proc_stdout = process.communicate()[0].strip()
-            #         # for line in proc_stdout.decode().split('\n'):
-            #         channel_send(self._logger, sendchannel, json.dumps({"response": proc_stdout.decode()}))
-            #
-            #     channel_log(self._logger, sendchannel, "<", message)
-            #     if isinstance(message, str) and message.startswith("ping"):
-            #         # pass
-            #         # reply
-            #         channel_send(self._logger, sendchannel, f'{{"ping":"pong","ts":"{datetime.utcnow()} UTC"}}')
-            #     if isinstance(message, str) and message.startswith('{"type'):
-            #         # reply
-            #         self.oncommand(self.gsdbs, json.loads(message))
-            #     if isinstance(message, str) and message.startswith('{"command'):
-            #         subprocess_cmd(json.loads(message)["command"])
+                @sendchannel.on("open")
+                def on_open():
+                    self._logger.info("datachannel open")
+
+                @sendchannel.on("message")
+                def on_message(message):
+                    def subprocess_cmd(command):
+                        process = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
+                        proc_stdout = process.communicate()[0].strip()
+                        # for line in proc_stdout.decode().split('\n'):
+                        channel_send(self._logger, sendchannel, json.dumps({"response": proc_stdout.decode()}))
+
+                    channel_log(self._logger, sendchannel, "<", message)
+                    if isinstance(message, str) and message.startswith("ping"):
+                        channel_send(self._logger, sendchannel, f'{{"ping":"pong","ts":"{datetime.utcnow()} UTC"}}')
+                    if isinstance(message, str) and message.startswith('{"type'):
+                        self.oncommand(self.gsdbs, json.loads(message))
+                    if isinstance(message, str) and message.startswith('{"command'):
+                        subprocess_cmd(json.loads(message)["command"])
 
             await pc.setLocalDescription(await pc.createOffer())
             await self.sio.emit("watcher", {"target": description,
@@ -419,13 +433,24 @@ class GSPeerConnectionBroadcasterUniversalNew:
         await self.sio.wait()
 
 
-async def createTerminatorView(gsdbs, aioqueue: asyncio.Queue, target, signalingserver=None
-                         ):
-    terminatorviewtask = await asyncio.gather(checkTerminatorQueue(aioqueue),
-                                              GSPeerConnectionBroadcasterUniversalNew.create(gsdbs=gsdbs,
-                                                                                             target=target + "terminatorview",
-                                                                                             videostream=False,
-                                                                                             videotrack=True,
-                                                                                             signalingserver=signalingserver
-                                                                                             ))
-    return terminatorviewtask
\ No newline at end of file
+async def createTerminatorView(gsdbs, aioqueue: asyncio.Queue,
+                               commandqueue: asyncio.Queue,
+                               target,
+                               signalingserver=None, oncommand=None):
+    datachannelist = []
+
+    def addDatachannel(channel):
+        datachannelist.append(channel)
+
+    terminatorviewtask = await asyncio.gather(
+        checkCommandQueue(commandqueue, datachannelist),
+        checkTerminatorQueue(aioqueue),
+        GSPeerConnectionBroadcasterUniversalNew.create(gsdbs=gsdbs,
+                                                       target=target + "terminatorview",
+                                                       videostream=False,
+                                                       videotrack=True,
+                                                       signalingserver=signalingserver,
+                                                       addChannel=addDatachannel,
+                                                       oncommand=oncommand
+                                                       ))
+    return terminatorviewtask
